/*
	Staunton Gangwar
	File Author: Force
*/

const CLASS_FIREMAN = 4;
const CLASS_POLICE = 5;
const CLASS_MEDIC = 6;

RandomSpawns <- [ Vector( 181.65, -470.1, 26.16 ), Vector( 414.4, -1389.5, 26.16 ), Vector( 68.09, -1547.3, 28.29 ), Vector( -16.11, -1018.3, 26.16 ) ];

function onScriptLoad()
{
	/* Lock the players on Staunton Island */
	SetBoundaries( Vector( -337.2, -1767.5, -1000.00 ), Vector( 557.5, 286.41, 1000.00 ) );
	CloseSSVBridge();
	SetSSVBridgeLock( true );
	
	/* Ammunation sphere, this can only be hit whilst the player is on foot */
	local pSphere = CreateSphere( Vector( 348.8, -709.4, 26.16 ), 3.00, Colour( 0, 0, 255 ) );
	if ( pSphere ) pSphere.Type = MARKER_TYPE_PLAYER;
	
	/* Radar blip to show where ammunation is */
	CreateBlip( BLIP_AMMU, Vector( 348.8, -709.4, 26.16 ) );
	
	dofile( "Scripts/StauntonGangwar/Ammunation.nut" );
	
	print( "[Loaded] Staunton Gangwar" );
	
	return 1;
}

function onScriptUnload()
{
	print( "[Unloaded] Staunton Gangwar" );
	
	return 1;
}

function onPlayerJoin( pPlayer )
{
	MessagePlayer( "Welcome to Staunton Gangwar Moneygrub!", pPlayer, Colour( 255, 200, 9 ) );
	
	return 1;
}

function onPlayerSpawn( pPlayer, pClass )
{
	if ( pClass.ID < CLASS_FIREMAN )
	{
		local iRandom = ( GetTickCount() % 4 );
		
		MessagePlayer( "You have spawned as a Robber.", pPlayer );
		pPlayer.Pos = RandomSpawns[ iRandom ];
	}
	if ( pClass.ID == CLASS_FIREMAN ) MessagePlayer( "You have spawned as a Fireman.", pPlayer );
	else if ( pClass.ID == CLASS_POLICE ) MessagePlayer( "You have spawned as a Cop.", pPlayer );
	else if ( pClass.ID == CLASS_MEDIC ) MessagePlayer( "You have spawned as a Medic.", pPlayer );
	
	GetOnlineRank();
	FindRankPos( pPlayer );
		
	return 1;
}

function onPlayerKill( pKiller, pPlayer, iWeapon, iBodyPart )
{
	local szWeapon = GetWeaponName( iWeapon ), szBodyPart = GetBodyPartName( iBodyPart ) == "Unknown" ? "Body" : GetBodyPartName( iBodyPart );
	Message( "** " + pKiller + " killed " + pPlayer + " (" + szWeapon + ") (" + szBodyPart + ")", Colour( 255, 0, 0 ) );
	
	/* Deal with the killer stuff */
	pKiller.Cash += 100;
	pKiller.Score = pKiller.Cash;
	if ( pKiller.Health < 100 ) pKiller.Health += 20;
	
	/* Deal with the player stuff */
	if ( pPlayer.Cash >= 100 ) pPlayer.Cash -= 100;
	else pPlayer.Cash = 0;

	pPlayer.Score = pPlayer.Cash;
	
	GetOnlineRank();
	FindRankPos( pKiller );
	FindRankPos( pPlayer );
	
	return 1;
}

function onPlayerDeath( pPlayer, iReason )
{
	Message( "** " + pPlayer + " has died.", Colour( 255, 0, 0 ) );
	
	/* Deal with the player stuff */
	if ( pPlayer.Cash >= 100 ) pPlayer.Cash -= 100;
	else pPlayer.Cash = 0;

	pPlayer.Score = pPlayer.Cash;
	
	GetOnlineRank();
	FindRankPos( pPlayer );
	
	return 1;
}

function onPlayerEnterSphere( pPlayer, pSphere )
{
	SendToAmmunation( pPlayer );
	
	return 1;
}

function onPlayerPart( pPlayer, iReason )
{
	CallClientFunc( pPlayer, "StauntonGangwar/Client.nut", "RemovePickup" );
	
	return 1;
}

/*
	Position calculator for the client-side script
*/

class PlrScore
{
	function constructor( ID = -1, Score = -1 ) { this.ID = ID; this.Score = Score; }
	
	ID = -1;
	Score = -1;
}

function FindRankPos( pPlayer )
{
	local iCounter = 0, iMax = OnlineRank.len(), pArray, iPos;
	
	while ( iCounter < iMax )
	{
		pArray = OnlineRank[ iCounter ];
		if ( pArray )
		{
			if ( pArray.ID == pPlayer.ID ) iPos = iCounter + 1;
		}
		iCounter ++;
	}
	
	CallClientFunc( pPlayer, "StauntonGangwar/Client.nut", "UpdateScore", iPos, pPlayer.Score );
}

function GetOnlineRank()
{
	InitializeArray()
	OnlineRank.sort( compare );
}

function compare( a, b )
{
	if ( a.Score < b.Score ) return 1;
	else if ( a.Score > b.Score ) return -1;
	else return 0;
}

function InitializeArray()
{
	local iMaxPlrs = GetMaxPlayers();
	local p, score;

	OnlineRank <- array( iMaxPlrs );

	for ( local i = 0; i < iMaxPlrs; i++ )
	{
		p = FindPlayer( i );

		if ( p ) 
		{
			OnlineRank[ i ] = PlrScore( p.ID, p.Score );
		}
		else OnlineRank[ i ] = PlrScore();
	}
}